import { Injectable } from '@angular/core';
import { Router } from '@angular/router';

import { Effect, Actions, ofType } from '@ngrx/effects';
import { Action } from '@ngrx/store';

import { Observable } from 'rxjs/Observable';
import { of } from 'rxjs/observable/of';

import 'rxjs/add/operator/catch';
import 'rxjs/add/operator/do';
import 'rxjs/add/operator/exhaustMap';
import 'rxjs/add/operator/switchMap';
import 'rxjs/add/operator/map';
import 'rxjs/add/operator/take';

import { CookieService } from 'ngx-cookie-service';

import { RPCClient } from 'packages/core/rpc/client/RPCClient';

import {
  Signin,
  SigninSuccess,
  SigninFailure,
  ActionType,
} from 'packages/member/store/auth';

@Injectable()
export class Effects {

  constructor(
    private actions$: Actions,
    private rpcClient: RPCClient,
    private cookieService: CookieService,
  ) { }

  @Effect({ dispatch: false })
  signinSuccess$ = this.actions$
    .ofType(ActionType.SigninSuccess)
    .do(
      () => {
        let queryString: string;
        if (this.cookieService.check('AuthToken')) {
          queryString = `AuthToken=${this.cookieService.get('AuthToken')}`;
        }
        this.rpcClient.connect(queryString);
      }
    );

}
