import { Action } from '@ngrx/store';

import { RESTError } from 'packages/core/rest/error';

import { DomainMember } from 'packages/domain/model';
import { Member } from '../../model';

export enum ActionType {
  Signin = '[member.auth] Signin',
  SigninSuccess = '[member.auth] SigninSuccess',
  SigninFailure = '[member.auth] SigninFailure',
  SigninRedirect = '[member.auth] SigninRedirect',
  Signout = '[member.auth] Signout',
  SignoutSuccess = '[member.auth] SignoutSuccess',
  SignoutFailure = '[member.auth] SignoutFailure',
}

export class Signin implements Action {
  readonly type = ActionType.Signin;

  constructor(public payload: {email: string, password: string, options: any}) {}
}

export class SigninSuccess implements Action {
  readonly type = ActionType.SigninSuccess;

  constructor(public payload: DomainMember) {}
}

export class SigninFailure implements Action {
  readonly type = ActionType.SigninFailure;

  constructor(public payload: RESTError) {}
}

export class SigninRedirect implements Action {
  readonly type = ActionType.SigninRedirect;
  constructor(public payload: string) {}
}

export class Signout implements Action {
  readonly type = ActionType.Signout;
}

export class SignoutSuccess implements Action {
  readonly type = ActionType.SignoutSuccess;
}

export class SignoutFailure implements Action {
  readonly type = ActionType.SignoutFailure;

  constructor(public payload: RESTError) {}
}

export type Actions =
  | Signin
  | SigninSuccess
  | SigninFailure
  | SigninRedirect
  | Signout
  | SignoutSuccess
  | SignoutFailure
;
